package ch.unibe.piccola.bridge;

import ch.unibe.piccola.vm.Form;
import ch.unibe.piccola.syntax.Syntax;

/**
 * Implements wrapper methods for primitive boolean operations.
 *
 * @author   Stefan Kneubuehl
 * @version  4.0 (Jun 09, 2003)
**/
public class BooleanOp
{

    // ------------------------------------------------------------------------
    // public static
    // ------------------------------------------------------------------------

    /**
     * Performs the logical <b>and</b> operation on the two specified boolean
     * values.
     *
     * @param  a left-hand argument for the operation.
     * @param  b right-hand argument for the operation.
     * @return <code>true</code>, if both <code>a</code> and <code>b</code>
     *         are <code>true</code>, <code>false</code> otherwise.
    **/
    public static boolean and(boolean a, boolean b)
    {
        return a && b;
    }

    /**
     * Performs the logical <b>not</b> operation on the specified boolean
     * value.
     *
     * @param  a argument for the operation.
     * @return <code>false</code>, if <code>a</code> is <code>true</code>,
     *         <code>true</code>, otherwise.
    **/
    public static boolean not(boolean a)
    {
        return !a;
    }

    /**
     * Performs a logical <b>or</b> operation on the two specified boolean
     * values.
     *
     * @param  a left-hand argument for the operation.
     * @param  b right-hand argument for the operation.
     * @return <code>false</code>, if both <code>a</code> and <code>b</code>
     *         are <code>false</code>, <code>true</code> otherwise.
    **/
    public static boolean or(boolean a, boolean b)
    {
        return a || b;
    }

    /**
     * Performs the <code>select</code> operation on the specified condition.
     * This method is used to implement conditional branching. The specified
     * cases must contain both a label <code>true</code> and
     * <code>false</code>.
     *
     * @param  condition the condition for the select operation.
     * @param  cases the cases for the select operation.
     * @return <code>cases.true</code>, if <code>condition</code> is
     *         <code>true</code>, <code>cases.false</code>, otherwise.
    **/
    public static Form select(boolean condition, Form cases)
    {
        return cases.project(condition ? LABEL_TRUE : LABEL_FALSE);
    }

    /**
     * Performs a logical <b>xor</b> operation on the two specified boolean
     * values.
     *
     * @param  a left-hand argument for the operation.
     * @param  b right-hand argument for the operation.
     * @return <code>false</code>, if both <code>a</code> and <code>b</code>
     *         have the same value, <code>true</code> otherwise.
    **/
    public static boolean xor(boolean a, boolean b)
    {
        return a ^ b;
    }

    // ------------------------------------------------------------------------
    // private static
    // ------------------------------------------------------------------------

    private static final String LABEL_TRUE = Syntax.labelTrue();

    private static final String LABEL_FALSE = Syntax.labelFalse();

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private BooleanOp()
    {
    }

}
