package ch.unibe.piccola.parser.javacc;

import java.util.ArrayList;
import java.util.Iterator;
import ch.unibe.piccola.syntax.Node;
import ch.unibe.piccola.source.SourceRef;

/**
 * @author   Stefan Kneubuehl
 * @version  3.4c (Jul 14, 2002)
**/
class BindingStatement extends Statement
{

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    public BindingStatement(Token def, ArrayList labels, Statement stmt)
    {
        super(stmt.getNode());
        m_def = def;
        m_labels = labels;
    }

    public Node extend(Node s)
    {
        return makeBinding(s);
    }

    public Node getNode()
    {
        return makeBinding(null);
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private Token m_def;

    private ArrayList m_labels;

    private Node makeBinding(Node s)
    {
        Token begin = null == m_def ? (Token) m_labels.get(0) : m_def;
        SourceRef end = (null == s ? super.getNode() : s).getSourceRef();
        SourceRef sc = new SourceRef(
            begin.m_beginPos, begin.beginLine, begin.beginColumn, end
        );
        String[] labels = tokensToStrings(m_labels);
        if (null == m_def && null == s && labels.length == 1) {
            return sf.newBinding(sc, super.getNode(), labels[0]);
        }
        else if (null != m_def && null == s && labels.length == 1) {
            return sf.newFixedPoint(sc, super.getNode(), labels[0]);
        }
        else {
            if (null == s) { s = sf.newEmpty(sc); }
            return sf.newCompoundBinding(
                sc, super.getNode(), s, labels, null != m_def
            );
        }
    }

    private String[] tokensToStrings(ArrayList tokens)
    {
        String[] labels = new String[tokens.size()];
        Iterator e = tokens.iterator();
        int i = 0;
        while (e.hasNext()) {
            Token token = (Token) e.next();
            labels[i] = null == token ? "" : token.toString();
            ++i;
        }
        return labels;
    }

}