package ch.unibe.piccola.source;

import java.io.IOException;
import java.io.Reader;

/**
 * This class represents a reference to a piece of source code.
 *
 * @author   Stefan Kneubuehl
 * @version  3.4c (Oct 02, 2002)
**/
public class SourceRef
{

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    public SourceRef(SourceRef begin, SourceRef end)
    {
        this(begin.m_source,
             begin.m_beginPos, begin.m_beginLine, begin.m_beginColumn,
             end.m_endPos, end.m_endLine, end.m_endColumn);
    }

    public SourceRef(SourceRef begin, int endPos, int endLine, int endColumn)
    {
        this(begin.m_source,
             begin.m_beginPos, begin.m_beginLine, begin.m_beginColumn,
             endPos, endLine, endColumn);
    }

    public SourceRef(int beginPos, int beginLine, int beginColumn,
                     SourceRef end)
    {
        this(end.m_source,
             beginPos, beginLine, beginColumn,
             end.m_endPos, end.m_endLine, end.m_endColumn);
    }

    public SourceRef(SourceCode source, int beginPos, int beginLine,
                     int beginColumn, int endPos, int endLine, int endColumn)
    {
        m_source = source;
        m_beginPos = beginPos;
        m_beginLine = beginLine;
        m_beginColumn = beginColumn;
        m_endPos = endPos;
        m_endLine = endLine;
        m_endColumn = endColumn;
    }

    /**
     * Returns the begin pos of this source code reference.
    **/
    public final int getBeginPos()
    {
        return m_beginPos;
    }

    /**
     * Returns the begin line of this source code reference.
    **/
    public final int getBeginLine()
    {
        return m_beginLine;
    }

    /**
     * Returns the begin column of this source code reference.
    **/
    public final int getBeginColumn()
    {
        return m_beginColumn;
    }

    /**
     * Returns the end pos of this source code reference.
    **/
    public final int getEndPos()
    {
        return m_endPos;
    }

    /**
     * Returns the end line of this source code reference.
    **/
    public final int getEndLine()
    {
        return m_endLine;
    }

    /**
     * Returns the begin line of this source code reference.
    **/
    public final int getEndColumn()
    {
        return m_endColumn;
    }

    /**
     * Returns a string containing the source code referenced by this
     * source code reference.
    **/
    public final String sourceCode() throws IOException
    {
        StringBuffer result = new StringBuffer(m_endPos - m_beginPos + 1);
        Reader reader = m_source.newReader();
        int pos = 0;
        for (; pos < m_beginPos; ++pos) {
            reader.read();
        }
        for (; pos < m_endPos; ++pos) {
            result.append((char) reader.read());
        }
        return result.toString();
    }

    /**
     * Returns a debug representation of this source code reference.
    **/
    public final String toString()
    {
        return m_source + " at line " + m_beginLine + ", column " +
            m_beginColumn;
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private final SourceCode m_source;

    private final int m_beginPos;

    private final int m_endPos;

    private final int m_beginLine;

    private final int m_beginColumn;

    private final int m_endLine;

    private final int m_endColumn;

}