package ch.unibe.piccola.source;

import java.io.InputStreamReader;
import java.io.IOException;
import java.io.Reader;
import java.net.URL;

/**
 * This class represents Piccola source code located by an URL.
 *
 * @author   Stefan Kneubuehl
 * @version  3.4c (Oct 17, 2002)
**/
public class URLSourceCode extends SourceCode
{

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    /**
     * Creates a new URLSourceCode object.
     *
     * @param url the url at which the Piccola source code is located.
    **/
    public URLSourceCode(URL url)
    {
        m_url = url;
    }

    /**
     * Creates a new Reader reading the source code.
     *
     * @return a new Reader reading the source code.
     * @throws IOException if an I/O error occurs while creating the Reader.
    **/
    public final Reader newReader() throws IOException
    {
        return new InputStreamReader(m_url.openStream());
    }

    /**
     * Returns a string representation of this source code.
     *
     * @return a string representation of this source code.
    **/
    public final String toString()
    {
        return m_url.toExternalForm();
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private final URL m_url;

}