package ch.unibe.piccola.syntax;

import ch.unibe.piccola.source.SourceRef;

/**
 * Represents an abstraction syntax node. The parameter may be a Piccola
 * identifier or the empty string (<code>""</code>) representing a void
 * parameter.
 *
 * @author   Stefan Kneubuehl
 * @version  3.4c (Nov 14, 2002)
**/
public class Abstraction extends OneChildNode
{

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    /**
     * Accepts a node visitor.
     *
     * @param  visitor the node visitor to be accepted.
     *
     * @return the result of the visitor's visitAbstraction method.
    **/
    public Object accept(NodeVisitor visitor)
    {
        return visitor.visitAbstraction(this);
    }

    /**
     * Compares this abstraction node with the specified object.
     *
     * @param  object the object to be compared with this node.
     *
     * @return <code>true</code> if this abstaction node is equal to object,
     *         <code>false</code> otherwise.
    **/
    public boolean equals(Object object)
    {
        if (!super.equals(object)) {
            return false;
        }
        Abstraction other = (Abstraction) object;
        return m_parameter.equals(other.m_parameter);
    }

    /**
     * Returns the formal parameter of this abstraction node. Returns either a
     * Piccola parameter or the empty string ("") denoting a void parameter.
     *
     * @return the parameter of this Abstraction node.
    **/
    public String getParameter()
    {
        return m_parameter;
    }

    /**
     * Returns a debug string representation of this abstraction node.
     *
     * @return a debug string representation of this abstraction node.
    **/
    public String toString()
    {
        return "Abstraction(" + m_parameter + ", " + getChild() + ")";
    }


    // ------------------------------------------------------------------------
    // default
    // ------------------------------------------------------------------------

    Abstraction(SourceRef sourceRef, Node child, String parameter)
    {
        super(sourceRef, child);
        m_parameter = parameter;
    }

    // ------------------------------------------------------------------------
    // protected
    // ------------------------------------------------------------------------

    protected Node newInstance(Node child)
    {
        return new Abstraction(getSourceRef(), child, m_parameter);
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private final String m_parameter;

}