package ch.unibe.piccola.syntax;

import ch.unibe.piccola.source.SourceRef;

/**
 * Represents a label node.
 *
 * @author   Stefan Kneubuehl
 * @version  3.4c (Nov 14, 2002)
**/
public class Label extends LeafNode
{

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    /**
     * Accepts a node visitor.
     *
     * @param  visitor the node visitor to be accepted.
     *
     * @return the result of the visitor's visitLabel method.
    **/
    public Object accept(NodeVisitor visitor)
    {
        return visitor.visitLabel(this);
    }

    /**
     * Compares this label node with the specified object.
     *
     * @param  object the object to be compared with this node.
     *
     * @return <code>true</code> if this label node is equal to object,
     *         <code>false</code> otherwise.
    **/
    public boolean equals(Object object)
    {
        if (!super.equals(object)) {
            return false;
        }
        Label other = (Label) object;
        return m_label.equals(other.m_label);
    }

    /**
     * Returns the label represented by this label node.
     *
     * @return the label represented by this label node.
    **/
    public String getLabel()
    {
        return m_label;
    }

    /**
     * Returns a debug string representation of this label node.
     *
     * @return a debug string representation of this label node.
    **/
    public String toString()
    {
        return "Label(" + m_label + ")";
    }

    // ------------------------------------------------------------------------
    // default
    // ------------------------------------------------------------------------

    Label(SourceRef sourceRef, String label)
    {
        super(sourceRef);
        m_label = label;
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private final String m_label;

}