package ch.unibe.piccola.syntax;

import ch.unibe.piccola.source.SourceRef;

/**
 * This class is the facade of the <code>ch.unibe.piccola.syntax</code>
 * package. It provides a corresponding <code>new</code>... method for
 * each syntax node type. The singleton instance of this class can be
 * accessed using the <code>INSTANCE</code> class attribute.
 *
 * @author   Stefan Kneubuehl
 * @version  3.4c (Oct 02, 2002)
**/
public class SyntaxFactory
{

    // ------------------------------------------------------------------------
    // public static
    // ------------------------------------------------------------------------

    /**
     * The singleton instance of this class.
    **/
    public static final SyntaxFactory INSTANCE = new SyntaxFactory();

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    public final Node newAbstraction(SourceRef sourceRef, Node child,
                                     String parameter)
    {
        return new Abstraction(sourceRef, child, parameter);
    }

    public final Node newApplication(SourceRef sourceRef,
                                     Node leftChild, Node rightChild)
    {
        return new Application(sourceRef, leftChild, rightChild);
    }

    public final Node newBinding(SourceRef sourceRef, Node child, String label)
    {
        return new Binding(sourceRef, child, label);
    }

    public final Node newCollection(SourceRef sourceRef, Node[] children,
                                    String opening, String closing)
    {
        return new Collection(sourceRef, children, opening, closing);
    }

    public final Node newCompoundBinding(SourceRef sourceRef, Node leftChild,
                                         Node rightChild, String[] labels,
                                         boolean fixedPoint)
    {
        return new CompoundBinding(
            sourceRef, leftChild, rightChild, labels, fixedPoint
        );
    }

    public final Node newConstant(SourceRef sourceRef, Object constant)
    {
        return new Constant(sourceRef, constant);
    }

    public final Node newCurriedAbstraction(SourceRef sourceRef, Node child,
                                            String[] params)
    {
        return new CurriedAbstraction(sourceRef, child, params);
    }

    public final Node newEmpty(SourceRef sourceRef)
    {
        return new Empty(sourceRef);
    }

    public final Node newExtension(SourceRef sourceRef,
                                   Node leftChild, Node rightChild)
    {
        return new Extension(sourceRef, leftChild, rightChild);
    }

    public final Node newFixedPoint(SourceRef sourceRef, Node child,
                                    String label)
    {
        return new FixedPoint(sourceRef, child, label);
    }

    public final Node newHide(SourceRef sourceRef, Node child, String label)
    {
        return new Hide(sourceRef, child, label);
    }

    public final Node newInfixApplication(SourceRef sourceRef, Node leftChild,
                                          Node rightChild, String label)
    {
        return new InfixApplication(sourceRef, leftChild, rightChild, label);
    }

    public final Node newLabel(SourceRef sourceRef, String label)
    {
        return new Label(sourceRef, label);
    }

    public final Node newNamedAbstraction(SourceRef sourceRef, Node leftChild,
                                          Node rightChild, String[] labels,
                                          String[] parameters,
                                          boolean fixedPoint)
    {
        return new NamedAbstraction(
            sourceRef, leftChild, rightChild, labels, parameters, fixedPoint);
    }

    public final Node newPrefixApplication(SourceRef sourceRef, Node child,
                                           String label)
    {
        return new PrefixApplication(sourceRef, child, label);
    }

    public final Node newProjection(SourceRef sourceRef, Node child,
                                    String label)
    {
        return new Projection(sourceRef, child, label);
    }

    public final Node newQuote(SourceRef sourceRef,
                               Node leftChild, Node rightChild)
    {
        return new Quote(sourceRef, leftChild, rightChild);
    }

    public final Node newRoot(SourceRef sourceRef)
    {
        return new Root(sourceRef);
    }

    public final Node newSandbox(SourceRef sourceRef,
                                 Node leftChild, Node rightChild)
    {
        return new Sandbox(sourceRef, leftChild, rightChild);
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private SyntaxFactory()
    {
    }

}