package ch.unibe.piccola.type;

import java.util.ArrayList;

/**
 * Represents either a provided type 'P' or a required type 'R'.
 *
 * @author    Stefan Kneubuehl
 * @version   3.4c (Jan 04, 2003)
**/
public class Type
{

    // ------------------------------------------------------------------------
    // public static
    // ------------------------------------------------------------------------

    /**
     * The empty type '()'.
    **/
    public static final Type EMPTY = TypeEmpty.INSTANCE;

    /**
     * The root type.
    **/
    public static final TypeVariable ROOT = TypeVariableRoot.INSTANCE;

    /**
     * Returns a new positive type variable.
    **/
    public static TypeVariableNormal newVariable()
    {
        return new TypeVariableNormal();
    }

    public static void resetVariables()
    {
        TypeVariableNormal.reset();
    }

    // ------------------------------------------------------------------------
    // protected static
    // ------------------------------------------------------------------------

    /**
     * Throws a type exception.
    **/
//    protected static Type labelException(String x)
//    {
//        throw new TypeException("Label " + x + " expected");
//    }

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    /**
     * Returns the required type 'this^type'.
    **/
    public Type and(Type type)
    {
        return equals(type) ? this : type.andLeft(this);
    }

    /**
     * Returns the type 'x:this'.
    **/
    public Type bind(String x)
    {
        return new TypeLabel(x, this);
    }

    /**
     * Returns the provided type 'this.type'.
    **/
    public Type extend(Type type)
    {
        return equals(type) ? this : type.extendLeft(this);
    }

    /**
     * Returns the provided type 'this\x'.
    **/
    public Type hide(String x)
    {
        return new TypeHide(x, this);
    }

    public boolean isVariable()
    {
        return false;
    }

    public boolean occurs(TypeVariable var)
    {
        return false;
    }

    /**
     * Returns this type with all type variables reversed.
    **/
    public Type reverse()
    {
        return this;
    }

    /**
     * Returns the type 'this->type'.
    **/
    public Type service(Type type)
    {
        return new TypeService(this, type);
    }

    /**
     * Simplifies the constraint provided '-> this
    **/
    public void simplifyConstraint(ArrayList list, Type provided)
    {
        if (provided instanceof TypeVariable) {
            for (int i = 0; i < list.size(); ++i) {
                Constraint c = (Constraint) list.get(i);
                if (c.getProvided() == provided) {
                    list.set(i, new Constraint(provided, c.getRequired().and(this)));
                    return;
                }
            }
        }
        list.add(new Constraint(provided, this));
    }

    /**
     * Substitutes var with replace in this type. Returns this[var -> replace].
    **/
    public Type subst(TypeVariable var, Type replace)
    {
        return this;
    }

    // ------------------------------------------------------------------------
    // protected
    // ------------------------------------------------------------------------

    protected Type()
    {
    }

    protected Type andLeft(Type left)
    {
        return new TypeAnd(left, this);
    }

    protected Type extendLeft(Type left)
    {
        return new TypeExtension(left, this);
    }

    protected Type lookupLabel(String x)
    {
        return null;
    }

    protected TypeService lookupService()
    {
        return null;
    }

    protected Type removeLabel(String x)
    {
        return this;
    }

    protected Type removeBindings()
    {
        return this;
    }

    protected Type removeService()
    {
        return this;
    }

}