package ch.unibe.piccola.type;

import java.util.ArrayList;

/**
 * Represents an required and type 'P^Q'.
 *
 * @author    Stefan Kneubuehl
 * @version   3.4c (Jan 03, 2003)
**/
public class TypeAnd extends Type
{

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    public boolean equals(Object object)
    {
        if (object instanceof TypeAnd) {
            TypeAnd other = (TypeAnd) object;
            return m_left.equals(other.m_left) &&
                   m_right.equals(other.m_right) &&
                   getClass().equals(other.getClass());
        }
        return false;
    }

    public Type getLeft()
    {
        return m_left;
    }

    public Type getRight()
    {
        return m_right;
    }

    /**
     * Implements type equivalence '(R^Q)\x = R\x ^ Q\x'.
    **/
    public Type hide(String x)
    {
        return doAnd(m_left.hide(x), m_right.hide(x));
    }

    public boolean occurs(TypeVariable var)
    {
        return m_left.occurs(var) || m_right.occurs(var);
    }

    public Type reverse()
    {
        return doAnd(m_left.reverse(), m_right.reverse());
    }

    public void simplifyConstraint(ArrayList list, Type provided)
    {
        m_left.simplifyConstraint(list, provided);
        m_right.simplifyConstraint(list, provided);
    }

    public Type subst(TypeVariable var, Type replace)
    {
        return doAnd(m_left.subst(var, replace), m_right.subst(var, replace));
    }

    public String toString()
    {
        return "(" + m_left + "^" + m_right + ")";
    }

    // ------------------------------------------------------------------------
    // default
    // ------------------------------------------------------------------------

    TypeAnd(Type left, Type right)
    {
        m_left = left;
        m_right = right;
    }

    // ------------------------------------------------------------------------
    // protected
    // ------------------------------------------------------------------------

    protected Type lookupLabel(String x)
    {
        throw new Error("Should not be called");
    }

    protected TypeService lookupService()
    {
        throw new Error("Should not be called");
    }

    protected Type removeBindings()
    {
        throw new Error("Should not be called");
    }

    protected Type removeLabel(String x)
    {
        throw new Error("Should not be called");
    }

    protected Type removeService()
    {
        throw new Error("Should not be called");
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private final Type m_left;

    private final Type m_right;

    private Type doAnd(Type left, Type right)
    {
        return (m_left == left && m_right == right) ? this : left.and(right);
    }

}