package ch.unibe.piccola.type;

import java.util.ArrayList;

/**
 * @author   Stefan Kneubuehl
 * @version  3.4c (Jan 04, 2003)
 *
 * Represents the type x:T
**/
public class TypeLabel extends Type
{

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    public boolean equals(Object object)
    {
        if (object instanceof TypeLabel) {
            TypeLabel other = (TypeLabel) object;
            return m_label.equals(other.m_label) &&
                   m_type.equals(other.m_type);
        }
        return false;
    }

    public String getLabel()
    {
        return m_label;
    }

    public Type getType()
    {
        return m_type;
    }

    /**
     * Implements type equivalences '(x:P)\x = ()' and '(y:P)\x = y:P'.
    **/
    public Type hide(String x)
    {
        return m_label.equals(x) ? Type.EMPTY : this;
    }

    public boolean occurs(TypeVariable var)
    {
        return m_type.occurs(var);
    }

    public Type reverse()
    {
        return doBind(m_type.reverse());
    }

    public void simplifyConstraint(ArrayList list, Type provided)
    {
        Type t = provided.lookupLabel(m_label);
        if (null == t) {
            super.simplifyConstraint(list, provided.removeBindings());
        }
        else {
            m_type.simplifyConstraint(list, t);
        }
    }

    public Type subst(TypeVariable var, Type replace)
    {
        return doBind(m_type.subst(var, replace));
    }

    public String toString()
    {
        return m_label + ":" + m_type.toString();
    }

    // ------------------------------------------------------------------------
    // default
    // ------------------------------------------------------------------------

    TypeLabel(String label, Type type)
    {
        m_label = label;
        m_type = type;
    }

    // ------------------------------------------------------------------------
    // protected
    // ------------------------------------------------------------------------

    /**
     * Implements type equivalence '(x:P).(x:R) = x:R'
    **/
    protected Type extendLeft(Type left)
    {
        Type t = left.removeLabel(m_label);
        return (left == t) ? super.extendLeft(t) : t.extend(this);
    }

    protected Type lookupLabel(String x)
    {
        return m_label.equals(x) ? m_type : null;
    }

    protected Type removeBindings()
    {
        return Type.EMPTY;
    }

    protected Type removeLabel(String x)
    {
        return m_label.equals(x) ? Type.EMPTY : this;
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private final String m_label;

    private final Type m_type;

    private Type doBind(Type type)
    {
        return (m_type == type) ? this : type.bind(m_label);
    }

}