package ch.unibe.piccola.type;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import ch.unibe.piccola.compiler.Simplifier;
import ch.unibe.piccola.parser.Parser;
import ch.unibe.piccola.parser.ParserException;
import ch.unibe.piccola.parser.ParserFactory;
import ch.unibe.piccola.source.StringSourceCode;
import ch.unibe.piccola.syntax.Node;

/**
 * @author    Stefan Kneubuehl
 * @version   3.4c (Jan 03, 2003)
**/
public class TypeTest extends TestCase {

    // ------------------------------------------------------------------------
    // test suite
    // ------------------------------------------------------------------------

    public TypeTest(String name)
    {
        super(name);
    }

    public static TestSuite suite()
    {
        return new TestSuite(TypeTest.class);
    }

    public static void main(String[] args)
    {
        junit.textui.TestRunner.run(suite());
    }

    // ------------------------------------------------------------------------
    // test environment
    // ------------------------------------------------------------------------

    public void setUp()
    {
        Type.resetVariables();
        a = Type.newVariable();
        b = Type.newVariable();
        c = Type.newVariable();
        xa = a.bind("x");
        xb = b.bind("x");
        yc = c.bind("y");
        sab = a.service(b);
        sbc = b.service(c);
    }

    private TypeVariable a, b, c;
    private Type sab, sbc, xa, xb, yc;

    private void assertType(Type t, String s)
    {
        assertEquals(s, t.toString());
    }

    // ------------------------------------------------------------------------
    // tests
    // ------------------------------------------------------------------------

    public void testEquivalence()
    {
        assertType(Type.EMPTY.extend(xa), "x:A+");
        assertType(xa.extend(Type.EMPTY), "x:A+");
        assertType(xa.extend(xb), "x:B+");
        assertType(xa.extend(yc), "(x:A+,y:C+)");
        assertType(sab.extend(sbc), "(B+->C+)");
        assertType(Type.EMPTY.and(xa), "x:A+");
        assertType(xa.and(Type.EMPTY), "x:A+");
        assertType(xa.extend(xa), "x:A+");
        assertType(xa.and(xa), "x:A+");
        assertType(xb.extend(yc).extend(xa), "(y:C+,x:A+)");
    }

    public void testRemoveLabel()
    {
        assertType(xa.removeLabel("x"), "()");
        assertType(xa.removeLabel("y"), "x:A+");
    }

    public void testExtend()
    {
        assertType(Type.ROOT.extend(xa).extend(xa), "(ROOT,x:A+)");
        assertType(Type.ROOT.extend(xa).extend(yc), "((ROOT,x:A+),y:C+)");
    }

    public void testLookupLabel()
    {
        assertType(xb.lookupLabel("x"), "B+");
        assertType(Type.EMPTY.bind("x").lookupLabel("x"), "()");
    }

    public void testSubst()
    {
        assertType(a.subst(a, xb), "x:B+");
    }

    public void testAnd()
    {
//        assertType(xa.and(xb), "x:(A+^B+)");
//        assertType(xa.and(yc), "(x:A+^y:C+)");
    }

}