package ch.unibe.piccola.vm;

import ch.unibe.piccola.code.CodeFrame;
import ch.unibe.piccola.source.SourceRef;

/**
 * This class is used by the VMCore to store code frames. It's faster than
 * the java.uti.Stack class. An default size of 32 has proven enough for
 * running all library tests.
 *
 * @author   Stefan Kneubuehl
 * @version  3.4c (Oct 16, 2002)
**/
class CodeFrameStack
{

    // ------------------------------------------------------------------------
    // private static
    // ------------------------------------------------------------------------

    private static final int INCREMENT = 32;

    // ------------------------------------------------------------------------
    // default
    // ------------------------------------------------------------------------

    CodeFrameStack()
    {
        m_size = INCREMENT;
        m_data = new CodeFrame[m_size];
        m_top = -1;
    }

    final boolean isEmpty()
    {
        return -1 == m_top;
    }

    final CodeFrame peek()
    {
        return m_data[m_top];
    }

    final CodeFrame pop()
    {
        return m_data[m_top--];
    }

    final void push(CodeFrame form)
    {
        ++m_top;
        if (m_top >= m_size - 1) {
            CodeFrame[] temp = new CodeFrame[m_size + INCREMENT];
            System.arraycopy(m_data, 0, temp, 0, m_size);
            m_size += INCREMENT;
            m_data = temp;
        }
        m_data[m_top] = form;
    }

    final int size()
    {
        return m_top + 1;
    }

    final SourceRef[] stackTrace()
    {
        SourceRef[] result = new SourceRef[m_top + 1];
        for (int i = 1; i <= m_top; ++i) {
            result[i] = m_data[i].getSourceRef();
        }
        return result;
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private CodeFrame[] m_data;

    private int m_top;

    private int m_size;

}

