package ch.unibe.piccola.vm;

import java.io.IOException;
import java.util.Iterator;
import java.util.SortedMap;
import ch.unibe.piccola.code.Code;
import ch.unibe.piccola.type.ContractualType;

/**
 * @author   Stefan Kneubuehl
 * @version  3.6c (Apr 23, 2003)
**/
public class ExtensionForm extends DefaultForm {

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    public boolean exists(String label)
    {
        return m_right.exists(label) || m_left.exists(label);
    }

    public Form hide(String label)
    {
        return m_left.hide(label).extend(m_right.hide(label));
    }

    public boolean isService()
    {
        return m_left.isService() || m_right.isService();
    }

    public boolean runtimeEquals(Form form)
    {
        Object object = down();
        if (this != object) {
            return object.equals(form.down());
        }
        if (isService()) {
            return false;
        }
        return getBindings().equals(form.getBindings());
    }

    public int runtimeHashCode()
    {
    	return m_left.hashCode() & m_right.hashCode();
    }

    public String runtimeToString()
    {
        Object object = down();
        if (this != object) {
            return object.toString();
        }
        StringBuffer buf = new StringBuffer("(");
        if (isService()) {
            buf.append("[service], ");
        }
        Iterator k = getBindings().keySet().iterator();
        if (k.hasNext()) {
            String key = (String)k.next();
            buf.append(key).append(" = ");
            buf.append(((Form) getBindings().get(key)).toString());
        }
        while (k.hasNext()) {
            buf.append(", ");
            String key = (String)k.next();
            buf.append(key).append(" = ");
            buf.append(((Form) getBindings().get(key)).toString());
        }
        return buf.append(")").toString();
    }

    public Service service()
    {
        return (m_right.isService() ? m_right : m_left).service();
    }

    public ContractualType type()
    {
        return m_left.type().extend(m_right.type());
    }

    public Object unwrap()
    {
        Object object = m_right.unwrap();
        if (object != m_right) {
            return object;
        }
        object = m_left.unwrap();
        if (object != m_left) {
            return object;
        }
        return this;
    }

    // ------------------------------------------------------------------------
    // default
    // ------------------------------------------------------------------------

    ExtensionForm(FormFactory factory, Form left, Form right)
    {
        super(factory);
        m_left = left;
        m_right = right;
    }

    // ------------------------------------------------------------------------
    // protected
    // ------------------------------------------------------------------------

    protected SortedMap buildBindings()
    {
        SortedMap bindings = super.buildBindings();
        bindings.putAll(m_left.getBindings());
        bindings.putAll(m_right.getBindings());
        return bindings;
    }

    public Form projectDef(String label, Form defaultForm)
    {
        Form result = m_right.projectDef(label, null);
        if (null == result) {
            result = m_left.projectDef(label, defaultForm);
        }
        return result;
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private final Form m_left;

    private final Form m_right;

}
