package ch.unibe.piccola.vm;

import java.lang.reflect.Array;
import ch.unibe.piccola.code.Arguments;
import ch.unibe.piccola.code.JavaOp;
import ch.unibe.piccola.syntax.Syntax;

/**
 * Piccola services that map to Java methods.
 *
 * @author   Stefan Kneubuehl
 * @version  3.6c (May 11, 2003)
**/
public class JavaArrayConstructor extends JavaOp
{

    // ------------------------------------------------------------------------
    // private static
    // ------------------------------------------------------------------------

    private static final String LABEL_NEWARRAY = Syntax.labelNewArray();

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    public Object call(Object target, Arguments args)
    {
        if (args.size() > 1) {
            throw new AgentException("Invalid number of arguments. 1 expected.");
        }
        if (!(args.get(0) instanceof Number)) {
            throw new AgentException("Invalid argument. Number expected.");
        }
        return Array.newInstance(m_class, ((Number) args.get(0)).intValue());
    }

    public int hashCode()
    {
        return m_class.hashCode();
    }

    public String toString()
    {
        return m_class.getName() + "." + LABEL_NEWARRAY;
    }

    // ------------------------------------------------------------------------
    // default
    // ------------------------------------------------------------------------

    JavaArrayConstructor(Class cl)
    {
        m_class = cl;
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private Class m_class;

}

