/* File: FormAsTree.java
 * Author: F. Achermann acherman@iam.unibe.ch
 * Intend:
 * History: 
 * 
 */

// package pi.piccola.bridge;

import pi.pil.*;
import javax.swing.tree.*;
import java.util.*; 

import javax.swing.event.TreeModelListener; 

// TODO: refactor the strange if then else cases

/** Wraps a Form (or a Form Value) as a swing tree model. 
 *  This code relies on Java2.  For Java 1.1.8 with older swing, 
 *  you have to adapt the initialize mehtod below(). 
 */
public class FormAsTree implements TreeModel {


    /** the root of this forms tree
     */
    protected final FormAsTree root; 

    protected final IForm form; 
    
    
    /** the label this (subform) is accessible; 
     */ 
    protected final String label; 
    
    /** contains the list of (distinct) keys in the form
     *  written by initialize()
     *  @see initialize()
     */
    protected String[] labels; 

    
    /** contains the list of values in the form
     *  null when the coresponding key label is not read
     */
    protected Object[] values; 

	protected String name;

    public FormAsTree(IForm form) {
        this.form = form; 
        this.root = this; 
        this.label = null;
        initialize(); 
    	Debug("new FormAsTree" + this);
    }


    public FormAsTree(String label,  FormAsTree root, IForm form) {
        this.form = form; 
        this.root = root; 
        this.label = label; 
        initialize(); 
    	Debug("new FormAsTree" + this);
    }



    /** get lazy information about form
     *  writes and initializes labels; 
     *  @see labels
     */
    protected void initialize() {
        Enumeration en = form.labels(); 
        HashSet keys = new HashSet(); 
        while (en.hasMoreElements()) keys.add(en.nextElement()); 
        
        labels = (String[])keys.toArray(new String[keys.size()]); 
        Arrays.sort(labels); 

//      USE THIS IF JDK 1.1.x
//      thanks to Oscar Nierstrasz.         
//        Arrays.sort(labels, new Comparator() {
//                public int compare(Object o1, Object o2) {
//                    return ((String)o1).compareTo((String)o2);
//                }});

   
        // for (int i=0; i<labels.length; i++) System.out.print(labels[i] + " "); 
        // System.out.println(); 
        
        values = new Object[labels.length]; 
        
        this.name = this.asString();
    }



    /** 
     * Adds a listener for the TreeModelEvent posted after the tree changes.
     * FormTrees are immutable since forms are. 
     */
    public  void addTreeModelListener(TreeModelListener l) {
        Debug(this + ".addTreeModelListener(" + l + ")"); 
    }

    /**
     *  Returns the child of parent at index index in the parent's child array.
     */
    public Object getChild(Object parent, int index) {
        Debug(this + ".getChild(" + parent + ", " + index + ")"); 
        if (parent == this) {
            if (values[index] == null) {
                // lazy init value
                Object v = form.project(labels[index]); 
                // Debug("Lookup of subform: " + v);
                Debug("Lookup of subform: " + labels[index]);
                if (v instanceof IForm) {
                    v = new FormAsTree(labels[index], root, (IForm)v); 
                }
                values[index] = v;
                return v; 
            } else {
                return values[index]; 
            }
        } else {
            if (parent instanceof TreeModel) {
                return ((TreeModel)parent).getChild(parent, index); 
            } else 
                return null; 
        
        }
    }
    
    /**
     *   Returns the number of children of parent.
     */
    public int getChildCount(Object parent) {
        Debug(this + ".getChildCount(" + parent + ")"); 
        if (parent == this) {
            return labels.length;         
        } else {
            if (parent instanceof TreeModel) {
                return ((TreeModel)parent).getChildCount(parent); 
            } else 
                return 0; 
        }
    }
    
    /**
     *  Returns the index of child in parent.
     */
    public int getIndexOfChild(Object parent, Object child) {
        Debug(this + ".getIndexOfChild(" + parent + ", " + child + ")"); 
        return 0; 
    }
    
    /**
     *  Returns the root of the tree.
     */
    public Object getRoot() {
        Debug(this + ".getRoot()");
        return root;  
    }
    
    /**
     *  Returns true if node is a leaf.
     */
    public boolean isLeaf(Object node) {
        Debug(this + ".isLeaf(" + node + ")");
        if (node == this) {
            return labels.length == 0;         
        } else {
            if (node instanceof TreeModel) {
                return ((TreeModel)node).isLeaf(node); 
            } else 
                return true; 
        }
    }


    public String toString() {
    	return this.name;
    }
    
    public String asString() {
        StringBuffer r = new StringBuffer(); 
        if (label != null) r.append(label + " "); 
        r.append("= "); 
        
        String f = form.toString(); 
        if (f.length() > 20) {
            r.append(f.substring(0, 20)); 
            r.append("..."); 
        } else 
            r.append(f); 
        return r.toString(); 
        
    }
    
    
    /**
     *  Removes a listener previously added with addTreeModelListener().
     */
    public void removeTreeModelListener(TreeModelListener l) {
        System.out.println(this + ".removeTreeModelListener(" + l + ")"); 
    }
    
    /**
     *  Messaged when the user has altered the value for the item identified by path to newValue.
     */
    public void valueForPathChanged(TreePath path, Object newValue) {
        System.out.println(this + ".valueForPathChanged(" + path + ", " + newValue + ")"); 
    }

	private void Debug(String Msg) {
		System.out.println(Msg);
	}

}
